@@ -32,27 +32,76 @@
 #include "mixgtk_colorsel.h"
 #include "mixgtk_fontsel.h"
 #include "mixgtk_config.h"
+#include "mixgtk_wm.h"
 #include "mixgtk.h"
 
+static const gchar *SPLIT_ARG_ = "-s";
+static const gchar *SPLIT_LONG_ARG_ = "--s";
+
+gboolean
+mixgtk_restart (void)
+{
+  mix_vm_t *vm;
+  
+  if (!mixgtk_widget_factory_init ())
+    {
+      g_error ("Unable to initialise application: missing glade file");
+      return FALSE;
+    }
+
+  mixgtk_fontsel_load_defaults ();
+
+  if (!mixgtk_cmd_dispatcher_init (MIXGTK_MAIN))
+    {
+      g_error ("Unable to initialise application (command dispatcher)\n");
+      return FALSE;
+    }
+
+  vm = mixgtk_cmd_dispatcher_get_vm ();
+  
+  if (!mixgtk_mixvm_init (vm, MIXGTK_MIXVM_DIALOG))
+    {
+      g_error ("Unable to initialise application (mixvm widgets)\n");
+      return FALSE;
+    }
+
+  if (!mixgtk_mixal_init (vm, MIXGTK_MIXAL_DIALOG))
+    {
+      g_error ("Unable to initialise application (mixal widgets)\n");
+      return FALSE;
+    }
+
+  if (!mixgtk_device_init (GTK_NOTEBOOK
+			   (mixgtk_widget_factory_get
+			    (MIXGTK_DEVICES_DIALOG, MIXGTK_WIDGET_DEVICE)),
+			   vm))
+    {
+      g_error ("Unable to initialise application (devices)\n");
+      return FALSE;
+    }
+  
+  if (!mixgtk_wm_init ())
+    {
+      g_error ("Unable to initialise application (visibility)\n");
+      return FALSE;
+    }
+
+  mixgtk_input_init ();
+  mixgtk_colorsel_init ();
+
+  gtk_widget_show (mixgtk_widget_factory_get_dialog (MIXGTK_MAIN));
+  mixgtk_mixvm_update_vm_widgets ();
+
+  return TRUE;
+}
 
 /* initialise the app */
 gboolean
 mixgtk_init (int argc, char *argv[])
 {
-  mix_vm_t *vm;
-
-  const gchar *gfile = GLADE_FILE;
-  FILE *f = fopen (gfile, "r");
-  if (!f && !(f = fopen ((gfile = LOCAL_GLADE_FILE), "r")))
-    {
-      g_error ("Configuration file not found (tried %s and %s)",
-	       GLADE_FILE, LOCAL_GLADE_FILE);
-      return FALSE;
-    }
-  fclose (f);
+  gboolean split = FALSE;
+  gint k;
   
-  gtk_init (&argc, &argv);
-
   mix_init_lib ();
   
   if (!mixgtk_config_load ())
@@ -61,52 +110,24 @@ mixgtk_init (int argc, char *argv[])
       return FALSE;
     }
   
-  if (!mixgtk_widget_factory_init (gfile))
-    {
-      g_error ("Unable to initialise application using %s\n", gfile);
-      return FALSE;
-    }
-
-  if (!mixgtk_cmd_dispatcher_init ())
-    {
-      g_error ("Unable to initialise application (command dispatcher)\n");
-      return FALSE;
-    }
-
-  vm = mixgtk_cmd_dispatcher_get_vm ();
+  for (k = 0; k < argc; ++k)
+    if (!strcmp (argv[k], SPLIT_ARG_) || !strcmp (argv[k], SPLIT_LONG_ARG_))
+      {
+	split = TRUE;
+	break;
+      }
+  if (!split && mixgtk_config_is_split ()) split = TRUE;
+  mixgtk_config_set_split (split);
   
-  if (!mixgtk_device_init (GTK_NOTEBOOK
-			   (mixgtk_widget_factory_get (MIXGTK_WIDGET_DEVICE)),
-			   vm))
-    {
-      g_error ("Unable to initialise application (devices)\n");
-      return FALSE;
-    }
+  gtk_init (&argc, &argv);
 
-  if (!mixgtk_mixvm_init (vm))
-    {
-      g_error ("Unable to initialise application (mixvm widgets)\n");
-      return FALSE;
-    }
-
-  if (!mixgtk_mixal_init (vm))
-    {
-      g_error ("Unable to initialise application (mixal widgets)\n");
-      return FALSE;
-    }
-
-  mixgtk_fontsel_load_defaults ();
-  
-  return TRUE;
+  return mixgtk_restart ();
 }
 
 /* main loop */
 void
 mixgtk_main (void)
 {
-  gtk_widget_show (mixgtk_widget_factory_get_dialog (MIXGTK_MAIN));
-  mixgtk_mixvm_update_vm_widgets ();
-
   gtk_main ();
 }
 
@@ -115,6 +136,6 @@ void
 mixgtk_release (void) 
 {
   if (mixgtk_config_is_autosave ()) mixgtk_config_save ();
-  
+  mix_vm_cmd_dispatcher_delete (mixgtk_cmd_dispatcher_get_mix_dispatcher ());
   mix_release_lib ();
 }
