@@ -29,12 +29,16 @@
 #include <mixlib/mix.h>
 
 /* the app files directory */
-#define MIXGTK_FILES_DIR  ".gmixvm"
+#define MIXGTK_FILES_DIR  ".mdk"
 
 /* initialise the app */
 extern gboolean
 mixgtk_init (int argc, char *argv[]);
 
+/* restart the gui */
+extern gboolean
+mixgtk_restart (void);
+
 /* enter the main mixgtk loop */
 extern void
 mixgtk_main (void);
