@@ -54,10 +54,20 @@ typedef struct mixgtk_dispatch_
   GtkWidget *status;		/* the status bar widget */
   guint context;		/* context of the status bar messages */
   GCompletion *completions;	/* mixvm command completions */
+  gchar *last_file;
 } mixgtk_dispatch_data_t;
 
 static struct mixgtk_dispatch_ dis_data_ = {NULL};
 
+static GtkWidget *ext_dlg_ = NULL;
+static GtkWidget *ed_entry_ = NULL;
+static GtkWidget *asm_entry_ = NULL;
+
+static const gchar *ED_NAME_ = "editor_entry";
+static const gchar *ASM_NAME_ = "mixasm_entry";
+
+static const gchar *TITLE_FORMAT_ = "gmixvm - %s";
+
 static void
 log_command_ (mixgtk_dispatch_data_t *dis, const gchar *cmd)
 {
@@ -118,7 +128,13 @@ load_post_hook_ (mix_vm_cmd_dispatcher_t *dis, const gchar *arg,
  
   if (mix_vm_cmd_dispatcher_get_last_result (dis))
     {
-      gchar *file = g_strdup (arg);
+      GtkWindow *mainw =
+	GTK_WINDOW (mixgtk_widget_factory_get_dialog (MIXGTK_MAIN));
+
+      if (dis_data_.last_file) g_free (dis_data_.last_file);
+      dis_data_.last_file = g_strdup_printf (TITLE_FORMAT_, arg);
+      gtk_window_set_title (mainw, dis_data_.last_file);
+
       mixgtk_mixal_load_file ();
       mixgtk_mixal_update ();
       mixgtk_mixal_update_bp_all ();
@@ -128,7 +144,7 @@ load_post_hook_ (mix_vm_cmd_dispatcher_t *dis, const gchar *arg,
 			      dis_data_.context, (guint)id);
       id =
 	gtk_statusbar_push (GTK_STATUSBAR (dis_data_.status), dis_data_.context,
-			    file);
+			    dis_data_.last_file);
     }
 }
 
@@ -203,10 +219,6 @@ install_hooks_ (void)
 /* configuration stuff */
 static const gchar *EDITOR_KEY_ = "Editor";
 static const gchar *MIXASM_KEY_ = "Mixasm";
-static const gchar *HISTORY_FILE_KEY_ = "History.file";
-static const gchar *MAX_HISTORY_KEY_ = "History.size";
-static const gchar *HISTORY_FILE_ = NULL;
-static int MAX_HISTORY_ = 50;
 
 static void
 read_config_ (void)
@@ -214,13 +226,6 @@ read_config_ (void)
   const gchar *editor = mixgtk_config_get (EDITOR_KEY_);
   const gchar *assem = mixgtk_config_get (MIXASM_KEY_);
 
-#ifdef HAVE_LIBHISTORY
-  const gchar *mh = mixgtk_config_get (MAX_HISTORY_KEY_);
-  if (mh) MAX_HISTORY_ = atoi (mh);
-  if (MAX_HISTORY_ <= 0) MAX_HISTORY_ = 50;
-  HISTORY_FILE_ = mixgtk_config_get (HISTORY_FILE_KEY_);
-#endif
-
   if (!editor)
     {
       static const gchar *ENV[] = {"MDK_EDITOR", "X_EDITOR", NULL};
@@ -241,13 +246,6 @@ read_config_ (void)
 
 }
 
-static GtkWidget *ext_dlg_ = NULL;
-static GtkWidget *ed_entry_ = NULL;
-static GtkWidget *asm_entry_ = NULL;
-
-static const gchar *ED_NAME_ = "editor_entry";
-static const gchar *ASM_NAME_ = "mixasm_entry";
-
 void
 on_external_programs_activate ()
 {
@@ -290,24 +288,45 @@ on_extern_ok_button_clicked ()
 
 /* initialise the command dispatcher */
 gboolean
-mixgtk_cmd_dispatcher_init (void)
+mixgtk_cmd_dispatcher_init (mixgtk_dialog_id_t top)
 {
-  if (!dis_data_.prompt)
+  static gboolean restart = FALSE;
+  gchar *text = NULL;
+
+  ext_dlg_ = NULL;
+  ed_entry_ = NULL;
+  asm_entry_ = NULL;
+
+  dis_data_.prompt =
+    mixgtk_widget_factory_get (top, MIXGTK_WIDGET_PROMPT);
+  g_return_val_if_fail (dis_data_.prompt != NULL, FALSE);
+  
+  if (dis_data_.log)
+    text = gtk_editable_get_chars (GTK_EDITABLE (dis_data_.log), 0, -1);
+  
+  dis_data_.log =
+    mixgtk_widget_factory_get (top, MIXGTK_WIDGET_LOG);
+  
+  g_return_val_if_fail (dis_data_.log != NULL, FALSE);
+  
+  if (text)
     {
-      dis_data_.prompt = mixgtk_widget_factory_get (MIXGTK_WIDGET_PROMPT);
-      g_return_val_if_fail (dis_data_.prompt != NULL, FALSE);
-      dis_data_.log = mixgtk_widget_factory_get (MIXGTK_WIDGET_LOG);
-      g_return_val_if_fail (dis_data_.log != NULL, FALSE);
+      gtk_text_insert (GTK_TEXT (dis_data_.log), NULL, NULL, NULL, text, -1);
+      g_free (text);
     }
-    
+  
   if (!dis_data_.dispatcher)
     {
+      static const gchar *HISTORY_FILE = "gmixvm.history";
+      static gint HISTORY_SIZE = 100;
+      mix_config_t *config = mixgtk_config_get_mix_config ();
+      
       int r = pipe (dis_data_.fildes);
       g_return_val_if_fail (r == 0, FALSE);
-      /* connect stdout/stderr to the pipe's write end */
+      /* connect stdout/stderr to the pipe's write end 
       if (dup2 (dis_data_.fildes[1], STDOUT_FILENO) == -1
 	  || dup2 (dis_data_.fildes[1], STDERR_FILENO) == -1)
-	return FALSE;
+	return FALSE;*/
       dis_data_.out = fdopen (dis_data_.fildes[1], "w");
       g_return_val_if_fail (dis_data_.out != NULL, FALSE);
       r = fcntl (dis_data_.fildes[0], F_GETFL, 0);
@@ -315,30 +334,36 @@ mixgtk_cmd_dispatcher_init (void)
       r |= O_NONBLOCK;
       r = fcntl(dis_data_.fildes[0], F_SETFL, r);
       g_return_val_if_fail (r != -1, FALSE);
+
+      if (!mix_config_get_history_file (config))
+	mix_config_set_history_file (config, HISTORY_FILE);
+      if (mix_config_get_history_size (config) == 0)
+	mix_config_set_history_size (config, HISTORY_SIZE);
+
       dis_data_.dispatcher =
-	mix_vm_cmd_dispatcher_new (dis_data_.out, dis_data_.out);
+	mix_vm_cmd_dispatcher_new_with_config (dis_data_.out,
+					       dis_data_.out,
+					       config);
       mix_vm_cmd_dispatcher_print_time (dis_data_.dispatcher, FALSE);
       
       install_hooks_ ();
     }
   
-  if (!dis_data_.status)
-    {
-      dis_data_.status = mixgtk_widget_factory_get (MIXGTK_WIDGET_STATUSBAR);
-      g_return_val_if_fail (dis_data_.status != NULL, FALSE);
-      dis_data_.context = gtk_statusbar_get_context_id (GTK_STATUSBAR
-							(dis_data_.status),
-							"cmd_dis_context");
-    }
-  
+  dis_data_.status =
+    mixgtk_widget_factory_get (MIXGTK_MAIN, MIXGTK_WIDGET_STATUSBAR);
+  g_return_val_if_fail (dis_data_.status != NULL, FALSE);
+  dis_data_.context = gtk_statusbar_get_context_id (GTK_STATUSBAR
+						    (dis_data_.status),
+						    "cmd_dis_context");
   if (!dis_data_.completions) init_completions_ ();
-  read_config_ ();
+  if (!restart) read_config_ ();
+  if (dis_data_.last_file)
+    gtk_window_set_title
+      (GTK_WINDOW (mixgtk_widget_factory_get_dialog (MIXGTK_MAIN)),
+       dis_data_.last_file);
 
-#ifdef HAVE_LIBHISTORY
-  using_history ();
-  stifle_history (MAX_HISTORY_);
-#endif
-  
+
+  restart = TRUE;
   return TRUE;
 }
 
@@ -381,6 +406,13 @@ mixgtk_cmd_dispatcher_get_src_path (void)
   return mix_vm_cmd_dispatcher_get_src_file_path (dis_data_.dispatcher);
 }
 
+/* get the mix cmd dispatcher */
+mix_vm_cmd_dispatcher_t *
+mixgtk_cmd_dispatcher_get_mix_dispatcher (void)
+{
+  return dis_data_.dispatcher;
+}
+
 /* process commands */
 void
 complete_command_ (void)
