@@ -33,6 +33,9 @@ static const gchar *MIXGTK_CONFIG_FILE_ = "gmixvm.config";
 static const gchar *SPLIT_KEY = "Split";
 static const gchar *SPLIT_YES = "Yes";
 static const gchar *SPLIT_NO = "No";
+static const gchar *SHOW_TB_KEY = "Toolbars";
+static const gchar *TB_YES = "Yes";
+static const gchar *TB_NO = "No";
 
 static mix_config_t *config_ = NULL;
 
@@ -103,3 +106,16 @@ mixgtk_config_set_split (gboolean split)
 {
   mixgtk_config_update (SPLIT_KEY, split? SPLIT_YES : SPLIT_NO);
 }
+
+gboolean
+mixgtk_config_show_toolbars (void) 
+{
+  const gchar *show = mixgtk_config_get (SHOW_TB_KEY);
+  return (!show || !strcmp (show, TB_YES));
+}
+
+void
+mixgtk_config_set_show_toolbars (gboolean show)
+{
+  mixgtk_config_update (SHOW_TB_KEY, show? TB_YES : TB_NO);
+}
