@@ -62,6 +62,11 @@ mixgtk_config_is_split (void);
 extern void
 mixgtk_config_set_split (gboolean split);
 
+extern gboolean
+mixgtk_config_show_toolbars (void);
+
+extern void
+mixgtk_config_set_show_toolbars (gboolean show);
 
 #endif /* MIXGTK_CONFIG_H */
 
