@@ -19,10 +19,32 @@
  *  
  */
 
-
 #include "mix_file.h"
 #include "xmix_device.h"
 
+/*
+  Set the directory for mix device files (by default, it's ".")
+  If the dir does not exist, it is created.
+*/
+gboolean
+mix_device_set_dir (const gchar *dirname)
+{
+  if (mix_stat_dir (dirname, "devices"))
+    {
+      if (DEV_DIR_) g_free (DEV_DIR_);
+      DEV_DIR_ = g_strdup (dirname);
+      return TRUE;
+    }
+  else
+    return FALSE;
+}
+
+const gchar *
+mix_device_get_dir (void)
+{
+  return DEV_DIR_;
+}
+
 mix_device_t *
 mix_device_new (mix_device_type_t type)
 {
