@@ -69,6 +69,16 @@ typedef enum {
   mix_dev_CHAR
 } mix_device_mode_t;
 
+/*
+  Set the directory for mix device files (by default, it's ".")
+  If the dir does not exist, it is created.
+*/
+extern gboolean
+mix_device_set_dir (const gchar *dirname);
+
+extern const gchar *
+mix_device_get_dir (void);
+
 /*
   Create a new device with default name and given type.
 */
