@@ -133,6 +133,16 @@ mix_file_to_FILE(const mix_file_t *file)
   return io_get_FILE_(file);
 }
 
+/* complete a name with an extension, if needed */
+gchar *
+mix_file_complete_name (const gchar *name, const gchar *extension)
+{
+  if (!name) return NULL;
+  if (!extension || !needs_completion_ (name, extension))
+    return g_strdup (name);
+  return add_completion_ (name, extension);
+}
+
 /* Get the base name and extension of file */
 const gchar *
 mix_file_base_name(const mix_file_t *file)
