@@ -27,10 +27,20 @@
 #include <ctype.h>
 #include <errno.h>
 
-#include "mix_vm_command.h"
+#include "mix_device.h"
 #include "mix_vm.h"
 #include "mix_vm_dump.h"
 #include "mix_eval.h"
+#include "mix_predicate.h"
+#include "mix_vm_command.h"
+
+#ifdef HAVE_LIBHISTORY
+#  include <readline/history.h>
+#endif
+
+#ifdef HAVE_LIBREADLINE
+#  include <readline/readline.h>
+#endif
 
 /* hooks */
 typedef struct 
@@ -45,6 +55,8 @@ typedef struct
   gpointer data;
 } global_hook_;
 
+#define PRNO_ MIX_PRED_MEM
+
 struct mix_vm_cmd_dispatcher_t 
 {
   mix_vm_t *vm;			/* the virtual machine */
@@ -65,6 +77,9 @@ struct mix_vm_cmd_dispatcher_t
   hook_ post_hooks[MIX_CMD_INVALID]; /* Post-command hooks */
   global_hook_ global_pre;	/* global pre-command hook */
   global_hook_ global_post;	/* global post-command hook */
+  mix_config_t *config;		/* externally provided configuration */
+  mix_predicate_t *preds[PRNO_]; /* predicates for conditional breakpoints */
+  GHashTable *mem_preds;	/* predicates for memory conditional bps */
 };
 
 /* command handlers */
@@ -94,10 +109,25 @@ DEC_FUN (cbpa_);
 DEC_FUN (cabp_);
 DEC_FUN (weval_);
 DEC_FUN (w2d_);
-DEC_FUN (tron_);
-DEC_FUN (troff_);
+DEC_FUN (tracing_);
+DEC_FUN (timing_);
 DEC_FUN (edit_);
 DEC_FUN (compile_);
+DEC_FUN (pedit_);
+DEC_FUN (sedit_);
+DEC_FUN (pasm_);
+DEC_FUN (sasm_);
+DEC_FUN (devdir_);
+DEC_FUN (sbpr_);
+DEC_FUN (sbpm_);
+DEC_FUN (sbpc_);
+DEC_FUN (sbpo_);
+DEC_FUN (cbpr_);
+DEC_FUN (cbpm_);
+DEC_FUN (cbpc_);
+DEC_FUN (cbpo_);
+DEC_FUN (pbt_);
+
 
 /* internal command info struct */
 typedef struct {
@@ -113,8 +143,14 @@ command_ commands_[] = {
   { "help", cmd_help_, N_("Display this text"), "help [COMMAND]"},
   { "load", cmd_load_, N_("Load a MIX code file"), "load FILENAME"},
   { "edit", cmd_edit_, N_("Edit a MIXAL source file"), "edit [FILENAME]"},
+  { "pedit", cmd_pedit_, N_("Print the external editor command"), "pedit"},
+  { "sedit", cmd_sedit_, N_("Set the external editor command"),
+    "sedit COMMAND (e.g. emacs %s)"},
   { "compile", cmd_compile_, N_("Compile a MIXAL source file"),
     "compile [FILENAME]"},
+  { "pasm", cmd_pasm_, N_("Print the compile command"), "pasm"},
+  { "sasm", cmd_sasm_, N_("Set the compile command"),
+    "sasm COMMAND (e.g. mixasm -g -l %s)"},
   { "run", cmd_run_, N_("Run loaded or given MIX code file"), 
     "run [FILENAME]"},
   { "next", cmd_next_, N_("Execute next instruction(s)"), 
@@ -141,15 +177,43 @@ command_ commands_[] = {
     "sbpa ADDRESS"},
   { "cbpa", cmd_cbpa_, N_("Clear break point at given address"),
     "cbpa ADDRESS"},
+  { "sbpr", cmd_sbpr_, N_("Set conditional breakpoint on register change"),
+    "sbpr A | X | J | I[1-6]"},
+  { "cbpr", cmd_cbpr_, N_("Clear conditional breakpoint on register change"),
+    "sbpr A | X | J | I[1-6]"},
+  { "sbpm", cmd_sbpm_, N_("Set conditional breakpoint on mem cell change"),
+    "sbpm ADDRESS"},
+  { "cbpm", cmd_cbpm_, N_("Clear conditional breakpoint on mem cell change"),
+    "cbpm ADDRESS"},
+  { "sbpc", cmd_sbpc_,
+    N_("Set conditional breakpoint on comparison flag change"), "sbpc"},
+  { "cbpc", cmd_cbpc_,
+    N_("Clear conditional breakpoint on comparison flag change"), "cbpc"},
+  { "sbpo", cmd_sbpo_,
+    N_("Set conditional breakpoint on overflow toggled"), "sbpo"},
+  { "cbpo", cmd_cbpo_,
+    N_("Set conditional breakpoint on overflow toggled"), "cbpo"},
   { "cabp", cmd_cabp_, N_("Clear all breakpoints"), "cabp"},
   { "weval", cmd_weval_, N_("Evaluate a given W-expression"), "weval WEXPR"},
   { "w2d", cmd_w2d_, N_("Convert a MIX word to its decimal value"),
     "w2d WORD"},
-  { "tron", cmd_tron_, N_("Turn on instruction tracing."), "tron"},
-  { "troff", cmd_troff_, N_("Turn off instruction tracing."), "troff"},
+  { "tracing", cmd_tracing_, N_("Turn on/off instruction tracing"),
+    "tracing [on|off]"},
+  { "pbt", cmd_pbt_, N_("Print backtrace of executed instructions"),
+    "pbt [INS_NO] (e.g pbt 5)"},
+  { "timing", cmd_timing_, N_("Turn on/off timing statistics"),
+    "timing [on|off]"},
+  { "devdir", cmd_devdir_, N_("Print/set devices directory"),
+    "devdir [NEWDIR]"},
   { NULL, NULL, NULL, NULL},
 };
 
+/* configuration keys */
+static const gchar *TRACING_KEY_ = "Tracing";
+static const gchar *TIMING_KEY_ = "Timing";
+static const gchar *EDITOR_KEY_ = "Editor";
+static const gchar *ASM_KEY_ = "Assembler";
+
 
 /* external interface */
 
@@ -218,20 +282,84 @@ mix_vm_cmd_dispatcher_new (FILE *out_fd, /* output messages file */
       result->pre_hooks[k].func = result->post_hooks[k].func = NULL;
       result->pre_hooks[k].data = result->post_hooks[k].data = NULL;
     }
-      
+  result->config = NULL;
+  
+  for (k = 0; k < PRNO_; ++k)
+    result->preds[k] = mix_predicate_new (k);
+  
+  result->mem_preds = g_hash_table_new (NULL, NULL);
+  
   return result;
 }
 
+mix_vm_cmd_dispatcher_t *
+mix_vm_cmd_dispatcher_new_with_config (FILE *out, FILE *err,
+				       mix_config_t *config)
+{
+  mix_vm_cmd_dispatcher_t *result = mix_vm_cmd_dispatcher_new (out, err);
+  if (result != NULL && (result->config = config) != NULL)
+    {
+      gint hsize = 0;
+      const gchar *val = mix_config_get (result->config, TRACING_KEY_);
+      if (val) cmd_tracing_ (result, val);
+      val = mix_config_get (result->config, EDITOR_KEY_);
+      if (val) mix_vm_cmd_dispatcher_set_editor (result, val);
+      val = mix_config_get (result->config, ASM_KEY_);
+      if (val) mix_vm_cmd_dispatcher_set_assembler (result, val);
+      val = mix_config_get (result->config, TIMING_KEY_);
+      if (val) cmd_timing_ (result, val);
+      val = mix_config_get_devices_dir (result->config);
+      if (!val || !mix_stat_dir (val, "devices"))
+	{
+	  gchar *dirname = g_dirname (mix_config_get_filename (config));
+	  cmd_devdir_ (result, dirname);
+	  g_free (dirname);
+	}
+      else
+	mix_device_set_dir (val);
+#ifdef HAVE_LIBHISTORY
+      val = mix_config_get_history_file (result->config);
+      hsize = mix_config_get_history_size (result->config);
+      using_history ();
+      stifle_history (hsize);
+      if (val)
+	{
+	  read_history ((char *)val);
+	  history_set_pos (history_base + history_length - 1);
+	}
+#endif
+    }
+  return result;
+}
+  
+
 /* delete (does not close the fds in the constructor) */
+static gboolean
+del_pred_ (gpointer key, gpointer val, gpointer data)
+{
+  if (val) mix_predicate_delete ((mix_predicate_t *)val);
+  return TRUE;
+}
+
 void
 mix_vm_cmd_dispatcher_delete (mix_vm_cmd_dispatcher_t *dis)
 {
+  const gchar *hfile = NULL;
+  gint k;
+  
   g_return_if_fail (dis != NULL);
   mix_eval_delete (dis->eval);
   mix_dump_context_delete (dis->dump);
   mix_vm_delete (dis->vm);
   if (dis->editor) g_free (dis->editor);
   if (dis->editor) g_free (dis->assembler);
+#ifdef HAVE_LIBHISTORY
+  if (dis->config && (hfile = mix_config_get_history_file
+		      (dis->config)))
+    write_history ((char *)hfile);
+#endif
+  for (k = 0; k < PRNO_; ++k) mix_predicate_delete (dis->preds[k]);
+  g_hash_table_foreach_remove (dis->mem_preds, del_pred_, NULL);
   g_free (dis);
 }
 
@@ -243,6 +371,8 @@ mix_vm_cmd_dispatcher_set_editor (mix_vm_cmd_dispatcher_t *dis,
   g_return_if_fail (dis != NULL);
   if (dis->editor) g_free (dis->editor);
   dis->editor = (edit_tplt) ? g_strdup (edit_tplt) : NULL;
+  if (dis->config && dis->editor)
+    mix_config_update (dis->config, EDITOR_KEY_, dis->editor);
 }
 
 void
@@ -252,6 +382,8 @@ mix_vm_cmd_dispatcher_set_assembler (mix_vm_cmd_dispatcher_t *dis,
   g_return_if_fail (dis != NULL);
   if (dis->assembler) g_free (dis->assembler);
   dis->assembler = (asm_tplt) ? g_strdup (asm_tplt) : NULL;
+  if (dis->config && dis->assembler)
+    mix_config_update (dis->config, ASM_KEY_, dis->assembler);
 }
 
 const gchar *
@@ -281,7 +413,7 @@ mix_vm_cmd_dispatcher_get_src_file_path (const mix_vm_cmd_dispatcher_t *dis)
       g_free (PATH);
       PATH = NULL;
     }
-    
+
   if (f)
     PATH = mix_file_complete_name (mix_src_file_get_path (f), MIX_SRC_DEFEXT);
       
@@ -504,10 +636,17 @@ cmd_help_ (mix_vm_cmd_dispatcher_t *dis, const gchar *arg)
      
   for (i = 0; commands_[i].name; i++)
     {
-      if (!*arg || (strcmp (arg, commands_[i].name) == 0))
+      if (!arg || !*arg)
 	{
-	  fprintf (dis->out ,_("%s\t\t%s. Usage: %s\n"), commands_[i].name, 
-		  _(commands_[i].doc), commands_[i].usage);
+	  fprintf (dis->out ,_("%s\t\t%s\n"), commands_[i].name, 
+		   _(commands_[i].doc));
+	  printed++;
+	}
+      else if ((strcmp (arg, commands_[i].name) == 0))
+	{
+	  fprintf (dis->out ,_("%s\t\t%s.\n\t\tUsage: %s\n"),
+		   commands_[i].name,  _(commands_[i].doc),
+		   commands_[i].usage);
 	  printed++;
 	}
     }
@@ -623,8 +762,6 @@ cmd_compile_ (mix_vm_cmd_dispatcher_t *dis, const gchar *arg)
     }
 }
 
-
-
 static gboolean
 cmd_run_ (mix_vm_cmd_dispatcher_t *dis, const gchar *arg)
 {
@@ -646,10 +783,23 @@ cmd_run_ (mix_vm_cmd_dispatcher_t *dis, const gchar *arg)
 	if (line != 0)
 	  fprintf (dis->out,
 		   _("... stopped: breakpoint at line %ld (address %d)\n"), 
-		  line, mix_vm_get_prog_count (dis->vm));
+		   line, mix_vm_get_prog_count (dis->vm));
 	else
 	  fprintf (dis->out, _("... stopped: breakpoint at address %d\n"),
-		  mix_vm_get_prog_count (dis->vm));
+		   mix_vm_get_prog_count (dis->vm));
+      }
+      break;
+    case MIX_VM_COND_BREAK:
+      {
+	gulong line = mix_vm_get_break_lineno (dis->vm);
+	if (line != 0)
+	  fprintf (dis->out, _("... stopped: %s (line %ld, address %d)\n"),
+		   mix_vm_get_last_breakpoint_message (dis->vm),
+		   line, mix_vm_get_prog_count (dis->vm));
+	else
+	  fprintf (dis->out, _("... stopped: %s (address %d)\n"),
+		   mix_vm_get_last_breakpoint_message (dis->vm),
+		   mix_vm_get_prog_count (dis->vm));
       }
       break;
     case MIX_VM_ERROR:
@@ -773,7 +923,7 @@ cmd_preg_ (mix_vm_cmd_dispatcher_t *dis, const gchar *arg)
 	    int i = arg[1] - '1';
 	    if ( i < 0 || i > 5 )
 	      {
-		fprintf (dis->err, _("Invalid I index: %d"), i);
+		fprintf (dis->err, _("Invalid I index: %d\n"), i);
 		return FALSE;
 	      }
 	    mix_dump_context_add_opt (dis->dump, opt[i]);
@@ -1269,17 +1419,319 @@ cmd_w2d_ (mix_vm_cmd_dispatcher_t *dis, const gchar *arg)
 }
 
 static gboolean
-cmd_tron_ (mix_vm_cmd_dispatcher_t *dis, const gchar *arg)
+cmd_tracing_ (mix_vm_cmd_dispatcher_t *dis, const gchar *arg)
 {
-  dis->trace = TRUE;
-  fprintf (dis->out, _("Instruction tracing has been turned ON.\n"));
+  static const gchar *ON = "on";
+  static const gchar *OFF = "off";
+  if (arg && !strcmp (arg, ON))
+    {
+      dis->trace = TRUE;
+      if (dis->config) mix_config_update (dis->config, TRACING_KEY_, ON);
+    }
+  else if (arg && !strcmp (arg, OFF))
+    {
+      dis->trace = FALSE;
+      if (dis->config) mix_config_update (dis->config, TRACING_KEY_, OFF);
+    }
+  else if (arg && !strlen (arg))
+    {
+      fprintf (dis->out, "Instruction tracing is currently set %s\n",
+	       dis->trace ? "ON" : "OFF");
+    }
+  else 
+    cmd_help_ (dis, "tracing");
   return TRUE;
 }
 
 static gboolean
-cmd_troff_ (mix_vm_cmd_dispatcher_t *dis, const gchar *arg)
+cmd_timing_ (mix_vm_cmd_dispatcher_t *dis, const gchar *arg)
 {
-  dis->trace = FALSE;
-  fprintf (dis->out, _("Instruction tracing has been turned OFF.\n"));
+  static const gchar *ON = "on";
+  static const gchar *OFF = "off";
+  if (arg && !strcmp (arg, ON))
+    {
+      dis->printtime = TRUE;
+      if (dis->config) mix_config_update (dis->config, TIMING_KEY_, ON);
+    }
+  else if (arg && !strcmp (arg, OFF))
+    {
+      dis->printtime = FALSE;
+      if (dis->config) mix_config_update (dis->config, TIMING_KEY_, OFF);
+    }
+  else if (arg && !strlen (arg))
+    {
+      print_time_ (dis);
+    }
+  else 
+    cmd_help_ (dis, "timing");
+  return TRUE;
+}
+  
+
+static gboolean
+cmd_pedit_ (mix_vm_cmd_dispatcher_t *dis, const gchar *arg)
+{
+  const gchar *ed = mix_vm_cmd_dispatcher_get_editor (dis);
+  if (dis)
+    fprintf (dis->out, "Edit command: %s\n", ed);
+  else
+    fprintf (dis->out, "Edit command not set (use sedit)\n");
+  return TRUE;
+}
+
+static gboolean
+cmd_sedit_ (mix_vm_cmd_dispatcher_t *dis, const gchar *arg)
+{
+  if (!arg || !strlen (arg)) return cmd_help_ (dis, "sedit");
+  mix_vm_cmd_dispatcher_set_editor (dis, arg);
+  return TRUE;
+}
+
+static gboolean
+cmd_pasm_ (mix_vm_cmd_dispatcher_t *dis, const gchar *arg)
+{
+  const gchar *ed = mix_vm_cmd_dispatcher_get_assembler (dis);
+  if (dis)
+    fprintf (dis->out, "Compile command: %s\n", ed);
+  else
+    fprintf (dis->out, "Compile command not set (use sasm)\n");
+  return TRUE;
+}
+
+static gboolean
+cmd_sasm_ (mix_vm_cmd_dispatcher_t *dis, const gchar *arg)
+{
+  if (!arg || !strlen (arg)) return cmd_help_ (dis, "sasm");
+  mix_vm_cmd_dispatcher_set_assembler (dis, arg);
+  return TRUE;
+}
+
+static gboolean
+cmd_devdir_ (mix_vm_cmd_dispatcher_t *dis, const gchar *arg)
+{
+  if (!arg || !strlen (arg))
+    fprintf (dis->out, "Device directory: %s\n", mix_device_get_dir ());
+  else if (mix_device_set_dir (arg) && dis->config)
+    mix_config_set_devices_dir (dis->config, arg);
+  return TRUE;
+}
+
+
+static const gint INVALID_REG_ = -2;
+
+static mix_predicate_type_t
+get_reg_pred_ (const gchar *arg)
+{
+  mix_predicate_type_t pred = INVALID_REG_;
+  
+  switch (*arg)
+    {
+    case 'A':
+      pred = MIX_PRED_REG_A;
+      break;
+    case 'X':
+      pred = MIX_PRED_REG_X;
+      break;
+    case 'J':
+      pred = MIX_PRED_REG_J;
+      break;
+    case 'I':
+      {
+	if ( strlen (arg) == 2 )
+	  {
+	    int i = arg[1] - '1';
+	    if (i >= 0 && i < 6)
+	      pred = MIX_PRED_REG_I1 - 1 + i;
+	  }
+      }
+      break;
+    default:
+      break;
+    }
+  return pred;
+}
+
+static gboolean
+cmd_sbpr_ (mix_vm_cmd_dispatcher_t *dis, const gchar *arg)
+{
+  mix_predicate_type_t pred = get_reg_pred_ (arg);
+  if (pred != INVALID_REG_)
+    {
+      mix_vm_set_conditional_breakpoint (dis->vm, dis->preds[pred]);
+      fprintf (dis->out, _("Conditional breakpoint on r%s change set\n"), arg);
+      return TRUE;
+    }
+  else
+    {
+      fprintf (dis->err, "Invalid argument %s\n", arg);
+      return FALSE;
+    }
+}
+
+static gboolean
+cmd_cbpr_ (mix_vm_cmd_dispatcher_t *dis, const gchar *arg)
+{
+  mix_predicate_type_t pred = get_reg_pred_ (arg);
+  if (pred != INVALID_REG_)
+    {
+      if (mix_vm_clear_conditional_breakpoint (dis->vm, dis->preds[pred]))
+	fprintf (dis->out,
+		 _("Conditional breakpoint on r%s change removed\n"),
+		 arg);
+      else
+	fprintf (dis->err, _("No breakpoint set on r%s change\n"), arg);
+      return TRUE;
+    }
+  else
+    {
+      fprintf (dis->err, _("Invalid argument %s\n"), arg);
+      return FALSE;
+    }
+}
+
+static gboolean
+cmd_sbpm_ (mix_vm_cmd_dispatcher_t *dis, const gchar *arg)
+{
+  gint add = atoi (arg);
+  gpointer key, value;
+  if (add < 0 || add > MIX_VM_CELL_NO)
+    {
+      fprintf (dis->out, _("Invalid memory address: %s\n"), arg);
+      return FALSE;
+    }
+  if (!g_hash_table_lookup_extended (dis->mem_preds, GINT_TO_POINTER (add),
+				     &key, &value))
+    {
+      mix_predicate_t *new_pred = mix_predicate_new (MIX_PRED_MEM);
+      mix_predicate_set_mem_address (new_pred, add);
+      g_hash_table_insert (dis->mem_preds,
+			   GINT_TO_POINTER (add), (gpointer)new_pred);
+      mix_vm_set_conditional_breakpoint (dis->vm, new_pred);
+    }
+  fprintf (dis->out, _("Conditional breakpoint on mem cell no. %d set\n"), add);
+  return TRUE;
+}
+
+static gboolean
+cmd_cbpm_ (mix_vm_cmd_dispatcher_t *dis, const gchar *arg)
+{
+  gint add = atoi (arg);
+  gpointer key, value;
+  if (add < 0 || add > MIX_VM_CELL_NO)
+    {
+      fprintf (dis->out, _("Invalid memory address: %s\n"), arg);
+      return FALSE;
+    }
+  if (g_hash_table_lookup_extended (dis->mem_preds, GINT_TO_POINTER (add),
+				    &key, &value))
+    {
+      g_hash_table_remove (dis->mem_preds, key);
+      mix_vm_clear_conditional_breakpoint (dis->vm, (mix_predicate_t *)value);
+      mix_predicate_delete ((mix_predicate_t *)value);
+      fprintf (dis->out,
+	       _("Conditional breakpoint on mem cell no. %d removed\n"), add);
+    }
+  else
+    {
+      fprintf (dis->out, _("No conditional breakpoint set at address %d\n"),
+	       add);
+    }
+  return TRUE;
+}
+
+static gboolean
+cmd_sbpo_ (mix_vm_cmd_dispatcher_t *dis, const gchar *arg)
+{
+  if (arg && strlen (arg))
+    fprintf (dis->err, _("Unexpected argument: %s\n"), arg);
+  else
+    {
+      mix_vm_set_conditional_breakpoint (dis->vm, dis->preds[MIX_PRED_OVER]);
+      fprintf (dis->out,
+	       _("Conditional breakpoint on overflow toggled set\n"));
+    }
+  return TRUE;
+}
+
+static gboolean
+cmd_cbpo_ (mix_vm_cmd_dispatcher_t *dis, const gchar *arg)
+{
+  if (arg && strlen (arg))
+    fprintf (dis->err, _("Unexpected argument: %s\n"), arg);
+  else
+    {
+      if (mix_vm_clear_conditional_breakpoint
+	  (dis->vm, dis->preds[MIX_PRED_OVER]))
+	fprintf (dis->out,
+		 _("Conditional breakpoint on overflow toggled removed.\n"));
+      else
+	fprintf (dis->err, _("No breakpoint set on overflow toggle\n"));
+    }
+  return TRUE;
+}
+
+static gboolean
+cmd_sbpc_ (mix_vm_cmd_dispatcher_t *dis, const gchar *arg)
+{
+  if (arg && strlen (arg))
+    fprintf (dis->err, _("Unexpected argument: %s\n"), arg);
+  else
+    {
+      mix_vm_set_conditional_breakpoint (dis->vm, dis->preds[MIX_PRED_CMP]);
+      fprintf (dis->out,
+	       _("Conditional breakpoint on comparison flag changed set.\n"));
+    }
+  return TRUE;
+}
+
+static gboolean
+cmd_cbpc_ (mix_vm_cmd_dispatcher_t *dis, const gchar *arg)
+{
+  if (arg && strlen (arg))
+    fprintf (dis->err, _("Unexpected argument: %s\n"), arg);
+  else
+    {
+      if (mix_vm_clear_conditional_breakpoint
+	  (dis->vm, dis->preds[MIX_PRED_CMP]))
+	fprintf
+	  (dis->out,
+	   _("Conditional breakpoint on comparison flag changed removed.\n"));
+      else
+	fprintf (dis->err, _("No breakpoint set on comparison flag change\n"));
+    }
+  return TRUE;
+}
+
+static gboolean
+cmd_pbt_ (mix_vm_cmd_dispatcher_t *dis, const gchar *arg)
+{
+  enum {SIZE = 256};
+  static gchar BUFFER[SIZE];
+  gint no = atoi (arg);
+  gint k = 0, address;
+  guint line;
+  const mix_src_file_t *file = mix_vm_get_src_file (dis->vm);
+  char *name = file ? g_basename (mix_src_file_get_path (file)) : NULL;
+
+  const GSList *add = mix_vm_get_backtrace (dis->vm);
+  while (add && (no == 0 || k < no))
+    {
+      BUFFER[0] = '\0';
+      address = GPOINTER_TO_INT (add->data);
+      line = mix_vm_get_address_lineno (dis->vm, address);
+      if (line && file)
+	{
+	  int j = 0;
+	  snprintf (BUFFER, SIZE, "%s", mix_src_file_get_line (file, line));
+	  while (!isspace (BUFFER[j])) j++;
+	  BUFFER[j] = '\0';
+	}
+      if (strlen (BUFFER) == 0)	snprintf (BUFFER, SIZE, "%d", address);
+      fprintf (dis->out, "#%d\t%s\tin %s%s:%d\n", k, BUFFER, name,
+	       MIX_SRC_DEFEXT, line);
+      ++k;
+      add = add->next;
+    }
+  /*  if (name) g_free (name); */
   return TRUE;
 }
