@@ -28,6 +28,7 @@
 #include <stdio.h>
 #include "mix.h"
 #include "mix_vm.h"
+#include "mix_config.h"
 
 /* mix_vm_cmd_dispatcher encapsulates a virtual machine and helper
    objects, providing a command driven interface with output to
@@ -40,7 +41,11 @@ typedef enum {
   MIX_CMD_HELP = 0,		/* echo help message */
   MIX_CMD_LOAD,			/* load a mix program */
   MIX_CMD_EDIT,			/* edit mixal source */
+  MIX_CMD_PEDIT,		/* print editor command */
+  MIX_CMD_SEDIT,		/* set editor command*/
   MIX_CMD_COMPILE,		/* compile mixal source */
+  MIX_CMD_PASM,			/* print compiler command */
+  MIX_CMD_SASM,			/* set assembler command */
   MIX_CMD_RUN,			/* run a loaded program */
   MIX_CMD_NEXT,			/* run next instruction */
   MIX_CMD_LOC,			/* print location pointer */
@@ -58,11 +63,21 @@ typedef enum {
   MIX_CMD_CBP,			/* clear breakpoint at lineno */
   MIX_CMD_SBPA,			/* set breakpoint at address */
   MIX_CMD_CBPA,			/* clear breakpoint at address */
+  MIX_CMD_SBPR,			/* set breakpoint on register changed */
+  MIX_CMD_CBPR,			/* clear breakpoint on register changed */
+  MIX_CMD_SBPM,			/* set breakpoint on mem cell changed */
+  MIX_CMD_CBPM,			/* clear breakpoint on mem cell changed */
+  MIX_CMD_SBPC,			/* set breakpoint on comp flag changed */
+  MIX_CMD_CBPC,			/* clear breakpoint on comp flag changed */
+  MIX_CMD_SBPO,			/* set breakpoint on overflow toggled */
+  MIX_CMD_CBPO,			/* clear breakpoint on overflow toggled */
   MIX_CMD_CABP,			/* clear all breakpoints */
   MIX_CMD_WEVAL,		/* evaluate a w-expression */
   MIX_CMD_W2D,			/* print word in decimal notation */
-  MIX_CMD_TRON,			/* enable instruction traces */
-  MIX_CMD_TROFF,		/* disble instruction traces */
+  MIX_CMD_TRACING,		/* enable/disable instruction traces */
+  MIX_CMD_PBT,			/* print backtrace */
+  MIX_CMD_TIMING,		/* enable/disable timing statistics */
+  MIX_CMD_DEVDIR,		/* print/set device directory */
   MIX_CMD_INVALID,		/* invalid command identifier */
 } mix_vm_command_t;
 
@@ -92,8 +107,12 @@ mix_vm_command_usage (mix_vm_command_t cmd);
 
 /* create a new command dispatcher */
 extern mix_vm_cmd_dispatcher_t *
-mix_vm_cmd_dispatcher_new (FILE *out_fd, /* output messages file */
-			   FILE *err_fd /* error messages file */);
+mix_vm_cmd_dispatcher_new (FILE *out, /* output messages file */
+			   FILE *err /* error messages file */);
+
+extern mix_vm_cmd_dispatcher_t *
+mix_vm_cmd_dispatcher_new_with_config (FILE *out, FILE *err,
+				       mix_config_t *config);
 
 /* delete (does not close the fds in the constructor) */
 extern void
