@@ -23,10 +23,10 @@
 #ifndef MIXVM_COMMAND_H
 #define MIXVM_COMMAND_H
 
-#include <glib.h>
+#include <mixlib/mix_config.h>
 
 extern void
-mixvm_cmd_init (char *arg, gboolean use_emacs);
+mixvm_cmd_init (mix_config_t *config, char *arg, gboolean use_emacs);
 
 extern gboolean
 mixvm_cmd_exec (char *line);
